#include "LPC13xx.h"
#include "core_cm3.h"
#include "type.h"

#include "i2c.h"
#include "uart.h"
#include "SystemTickTimer.h"

#include "MPU9150.h"

#include "usb.h"
#include "usbcfg.h"
#include "usbhw.h"
#include "usbcore.h"

#include "cdc.h"
#include "cdcuser.h"

typedef struct {
	uint8_t d[50];
	int num;
}GETDAT_T;




GETDAT_T USBdata;

/*----------------------------------------------------------------------------
  Init IO
 *---------------------------------------------------------------------------*/
void Init_IO (void)
{
	LPC_IOCON->PIO2_0 &= ~0xc0; //Init LED
	LPC_GPIO2->DIR |= 0x01;
}

/*----------------------------------------------------------------------------
  LED Control
 *---------------------------------------------------------------------------*/
void LED(int state)
{
	if(state == 1){
		LPC_GPIO2->DATA |= 0x01; //LED on
	}
	else{
		LPC_GPIO2->DATA &= ~0x01; //LED off
	}
}

/*----------------------------------------------------------------------------
 Initializes the VCOM port.
 Call this function before using VCOM_putchar or VCOM_getchar
 *---------------------------------------------------------------------------*/
void VCOM_Init(void)
{
	CDC_Init ();
}
/*----------------------------------------------------------------------------
  Reads character from serial port buffer and writes to USB buffer
 *---------------------------------------------------------------------------*/
void VCOM_Serial2Usb(GETDAT_T dat)
{
	if (dat.num!=0 && CDC_DepInEmpty) { // 1 is empty so if empty
		CDC_DepInEmpty = 0;
		USB_WriteEP (CDC_DEP_IN, (unsigned char* )&dat.d[0], dat.num);
	}
}
/*----------------------------------------------------------------------------
  checks the serial state and initiates notification
 *---------------------------------------------------------------------------*/
void VCOM_CheckSerialState (void)
{
	unsigned short temp;
	static unsigned short serialState;

	temp = CDC_GetSerialState();
	if (serialState != temp)
	{
		serialState = temp;
		CDC_NotificationIn(); // send SERIAL_STATE notification
	}
}

/*----------------------------------------------------------------------------
  Init Timer32_0
 *---------------------------------------------------------------------------*/
void Init_Timer32_0 (void)
{
	NVIC_EnableIRQ(TIMER_32_0_IRQn);
	NVIC_SetPriority(TIMER_32_0_IRQn, 2);

    LPC_SYSCON->SYSAHBCLKCTRL |= (1<<9);
    LPC_TMR32B0->PR =7200-1;
	LPC_TMR32B0->MCR =0x600;
	LPC_TMR32B0->MR3 = 100;
	LPC_TMR32B0->TCR =2;        // TCR Reset
}
/*----------------------------------------------------------------------------
  Init Timer32_1
 *---------------------------------------------------------------------------*/
void Init_Timer32_1 (void)
{
	NVIC_EnableIRQ(TIMER_32_1_IRQn);
	NVIC_SetPriority(TIMER_32_1_IRQn, 2);

	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<10);
	LPC_TMR32B1->PR  = 7200-1;
	LPC_TMR32B1->MCR = 0x600;
	LPC_TMR32B1->MR3 = 100;
	LPC_TMR32B1->TCR = 2;      // TCR Reset
}

/*----------------------------------------------------------------------------
  get data from sensor and making USBdata
 *---------------------------------------------------------------------------*/
void getDataSen(void)
{
	static uint8_t timestamp;

	USBdata.num  = 28;
	USBdata.d[0] = 0xff;
    USBdata.d[1] = 0xff;
    USBdata.d[2] = 0x52;
    USBdata.d[3] = 0x54;
    USBdata.d[4] = 0x39;
    USBdata.d[5] = 0x41;
	USBdata.d[6] = 0x05;       //2015/06/08 ver
	USBdata.d[7] = timestamp;

	if(timestamp == 0xff) timestamp = 0;
	else timestamp++;

	getDataFromMPU6050( (uint8_t *)(&USBdata.d[8]) );
	wait1usec(100);
	getDataFromAK8975( (uint8_t *)(&USBdata.d[22]) );

    return;
}

/*----------------------------------------------------------------------------
  Timer32bit timer0
 *---------------------------------------------------------------------------*/
void TIMER32_0_IRQHandler(void)
{
	static volatile uint32_t count;

	LPC_TMR32B0->IR=0x08; //clear interrupt flag

	//LED emitting fast
	count ++;
	if(count < 10 ) LED(1);
	else LED(0);
	if(count >20) count = 0;

	getDataSen();

	LPC_UART->IER = IER_THRE | IER_RLS;             // Disable RBR
	UARTSend( (uint8_t *)USBdata.d, USBdata.num );
	LPC_UART->IER = IER_THRE | IER_RLS | IER_RBR;   // Re-enable RBR
}

/*----------------------------------------------------------------------------
  Timer32bit timer1
 *---------------------------------------------------------------------------*/
void TIMER32_1_IRQHandler(void)
{
	static volatile uint32_t count;

	LPC_TMR32B1->IR=0x08; //clear interrupt flag

	//LED emitting slow
	count ++;
	if(count < 30 ) LED(1);
	else LED(0);

	if(count >60) count = 0;
	getDataSen();

	LPC_UART->IER = IER_THRE | IER_RLS;			    // Disable RBR
	UARTSend( (uint8_t *)USBdata.d, USBdata.num );
	LPC_UART->IER = IER_THRE | IER_RLS | IER_RBR;	// Re-enable RBR
	VCOM_Serial2Usb(USBdata); // read serial port and initiate USB event
	VCOM_CheckSerialState();
}

/*----------------------------------------------------------------------------
  Main
 *---------------------------------------------------------------------------*/
int main (void)
{
	volatile unsigned long i = 0;

	wait1msec(200);

	LPC_SYSCON->SYSAHBCLKCTRL |= ((1<<16) | (1<<14));  //EN IOCON, EN USBREG

	Init_IO();               // IO Initialization
	LED(1);

	USBIOClkConfig();
	Init_Timer32_0();        // TIMER32_0 Initialization
	Init_Timer32_1();        // TIMER32_1 Initialization
	VCOM_Init();             // VCOM Initialization
	USB_Init();              // USB Initialization
	UARTInit(57600);

	USB_Connect(TRUE);       // USB Connect
	while (!USB_Configuration) {
		if(i >= 5000){
			while (!I2CInit((uint32_t)I2CMASTER)); // wait until I2C is configured
			wait1usec(1000);
			initMPU6050();
			initAK8975();
			wait1usec(1000);
			LPC_TMR32B0->TCR = 1;    // start timer0 interrupt
			USB_Connect(FALSE);
			while(1);
		}
		wait1usec(1000);
		i++;
	}; // wait until USB is configured

	while (!I2CInit((uint32_t)I2CMASTER)); // wait until I2C is configured
	wait1usec(1000);
	initMPU6050();
	initAK8975();
	wait1usec(1000);

	LPC_TMR32B0->TCR = 0;                  // stop  timer0 interrupt
	LPC_TMR32B1->TCR = 1;                  // start timer1 interrupt

	while(1); //Infinite loop

	return 0;
}
/*----------------------------------------------------------------------------
  End
 *---------------------------------------------------------------------------*/
